searchData={"items":[{"type":"module","title":"PhoenixHTMLHelpers","doc":"Collection of helpers to generate and manipulate HTML contents.\n\nThese helpers were used in Phoenix v1.6 and earlier versions,\nbefore the introduction of `Phoenix.Component`.\n\nReplace `use Phoenix.HTML` in your applications by:\n\n```elixir\nimport Phoenix.HTML\nimport Phoenix.HTML.Form\nuse PhoenixHTMLHelpers\n```\n\nTo preserve backwards compatibility.","ref":"PhoenixHTMLHelpers.html"},{"type":"module","title":"PhoenixHTMLHelpers.Form","doc":"Functions for generating forms and inputs HTML.","ref":"PhoenixHTMLHelpers.Form.html"},{"type":"function","title":"PhoenixHTMLHelpers.Form.checkbox/3","doc":"Generates a checkbox.\n\nThis function is useful for sending boolean values to the server.","ref":"PhoenixHTMLHelpers.Form.html#checkbox/3"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Form.checkbox/3","doc":"# Assuming form contains a User schema\n    checkbox(form, :famous)\n    #=>  \n    #=>","ref":"PhoenixHTMLHelpers.Form.html#checkbox/3-examples"},{"type":"function","title":"Options - PhoenixHTMLHelpers.Form.checkbox/3","doc":"* `:checked_value` - the value to be sent when the checkbox is checked.\n    Defaults to \"true\"\n\n  * `:hidden_input` - controls if this function will generate a hidden input\n    to submit the unchecked value or not. Defaults to \"true\"\n\n  * `:unchecked_value` - the value to be sent when the checkbox is unchecked,\n    Defaults to \"false\"\n\n  * `:value` - the value used to check if a checkbox is checked or unchecked.\n    The default value is extracted from the form data if available\n\nAll other options are forwarded to the underlying HTML tag.","ref":"PhoenixHTMLHelpers.Form.html#checkbox/3-options"},{"type":"function","title":"Hidden fields - PhoenixHTMLHelpers.Form.checkbox/3","doc":"Because an unchecked checkbox is not sent to the server, Phoenix\nautomatically generates a hidden field with the unchecked_value\n*before* the checkbox field to ensure the `unchecked_value` is sent\nwhen the checkbox is not marked. Set `hidden_input` to false If you\ndon't want to send values from unchecked checkbox to the server.","ref":"PhoenixHTMLHelpers.Form.html#checkbox/3-hidden-fields"},{"type":"function","title":"PhoenixHTMLHelpers.Form.color_input/3","doc":"Generates a color input.\n\nSee `text_input/3` for example and docs.","ref":"PhoenixHTMLHelpers.Form.html#color_input/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.date_input/3","doc":"Generates a date input.\n\nSee `text_input/3` for example and docs.","ref":"PhoenixHTMLHelpers.Form.html#date_input/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.date_select/3","doc":"Generates select tags for date.\n\nWarning: This functionality is best provided by browsers nowadays.\nConsider using `date_input/3` instead.\n\nCheck `datetime_select/3` for more information on options and supported values.","ref":"PhoenixHTMLHelpers.Form.html#date_select/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.datetime_local_input/3","doc":"Generates a datetime-local input.\n\nSee `text_input/3` for example and docs.","ref":"PhoenixHTMLHelpers.Form.html#datetime_local_input/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.datetime_select/3","doc":"Generates select tags for datetime.\n\nWarning: This functionality is best provided by browsers nowadays.\nConsider using `datetime_local_input/3` instead.","ref":"PhoenixHTMLHelpers.Form.html#datetime_select/3"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Form.datetime_select/3","doc":"# Assuming form contains a User schema\n    datetime_select form, :born_at\n    #=>  ...  /\n    #=>  ...  /\n    #=>  ...  —\n    #=>  ...  :\n    #=>  ... \n\nIf you want to include the seconds field (hidden by default), pass `second: []`:\n\n    # Assuming form contains a User schema\n    datetime_select form, :born_at, second: []\n\nIf you want to configure the years range:\n\n    # Assuming form contains a User schema\n    datetime_select form, :born_at, year: [options: 1900..2100]\n\nYou are also able to configure `:month`, `:day`, `:hour`, `:minute` and\n`:second`. All options given to those keys will be forwarded to the\nunderlying select. See `select/4` for more information.\n\nFor example, if you are using Phoenix with Gettext and you want to localize\nthe list of months, you can pass `:options` to the `:month` key:\n\n    # Assuming form contains a User schema\n    datetime_select form, :born_at, month: [\n      options: [\n        {gettext(\"January\"), \"1\"},\n        {gettext(\"February\"), \"2\"},\n        {gettext(\"March\"), \"3\"},\n        {gettext(\"April\"), \"4\"},\n        {gettext(\"May\"), \"5\"},\n        {gettext(\"June\"), \"6\"},\n        {gettext(\"July\"), \"7\"},\n        {gettext(\"August\"), \"8\"},\n        {gettext(\"September\"), \"9\"},\n        {gettext(\"October\"), \"10\"},\n        {gettext(\"November\"), \"11\"},\n        {gettext(\"December\"), \"12\"},\n      ]\n    ]\n\nYou may even provide your own `localized_datetime_select/3` built on top of\n`datetime_select/3`:\n\n    defp localized_datetime_select(form, field, opts \\\\ []) do\n      opts =\n        Keyword.put(opts, :month, options: [\n          {gettext(\"January\"), \"1\"},\n          {gettext(\"February\"), \"2\"},\n          {gettext(\"March\"), \"3\"},\n          {gettext(\"April\"), \"4\"},\n          {gettext(\"May\"), \"5\"},\n          {gettext(\"June\"), \"6\"},\n          {gettext(\"July\"), \"7\"},\n          {gettext(\"August\"), \"8\"},\n          {gettext(\"September\"), \"9\"},\n          {gettext(\"October\"), \"10\"},\n          {gettext(\"November\"), \"11\"},\n          {gettext(\"December\"), \"12\"},\n        ])\n\n      datetime_select(form, field, opts)\n    end","ref":"PhoenixHTMLHelpers.Form.html#datetime_select/3-examples"},{"type":"function","title":"Options - PhoenixHTMLHelpers.Form.datetime_select/3","doc":"* `:value` - the value used to select a given option.\n    The default value is extracted from the form data if available.\n\n  * `:default` - the default value to use when none was given in\n    `:value` and none is available in the form data\n\n  * `:year`, `:month`, `:day`, `:hour`, `:minute`, `:second` - options passed\n    to the underlying select. See `select/4` for more information.\n    The available values can be given in `:options`.\n\n  * `:builder` - specify how the select can be build. It must be a function\n    that receives a builder that should be invoked with the select name\n    and a set of options. See builder below for more information.","ref":"PhoenixHTMLHelpers.Form.html#datetime_select/3-options"},{"type":"function","title":"Builder - PhoenixHTMLHelpers.Form.datetime_select/3","doc":"The generated datetime_select can be customized at will by providing a\nbuilder option. Here is an example from EEx:\n\n      %>\n      Date:   /   /  \n      Time:   :  \n     \n\nAlthough we have passed empty lists as options (they are required), you\ncould pass any option there and it would be given to the underlying select\ninput.\n\nIn practice, we recommend you to create your own helper with your default\nbuilder:\n\n    def my_datetime_select(form, field, opts \\\\ []) do\n      builder = fn b ->\n        assigns = %{b: b}\n\n        ~H\"\"\"\n        Date:   /   /  \n        Time:   :  \n        \"\"\"\n      end\n\n      datetime_select(form, field, [builder: builder] ++ opts)\n    end\n\nThen you are able to use your own datetime_select throughout your whole\napplication.","ref":"PhoenixHTMLHelpers.Form.html#datetime_select/3-builder"},{"type":"function","title":"Supported date values - PhoenixHTMLHelpers.Form.datetime_select/3","doc":"The following values are supported as date:\n\n  * a map containing the `year`, `month` and `day` keys (either as strings or atoms)\n  * a tuple with three elements: `{year, month, day}`\n  * a string in ISO 8601 format\n  * `nil`","ref":"PhoenixHTMLHelpers.Form.html#datetime_select/3-supported-date-values"},{"type":"function","title":"Supported time values - PhoenixHTMLHelpers.Form.datetime_select/3","doc":"The following values are supported as time:\n\n  * a map containing the `hour` and `minute` keys and an optional `second` key (either as strings or atoms)\n  * a tuple with three elements: `{hour, min, sec}`\n  * a tuple with four elements: `{hour, min, sec, usec}`\n  * `nil`","ref":"PhoenixHTMLHelpers.Form.html#datetime_select/3-supported-time-values"},{"type":"function","title":"PhoenixHTMLHelpers.Form.email_input/3","doc":"Generates an email input.\n\nSee `text_input/3` for example and docs.","ref":"PhoenixHTMLHelpers.Form.html#email_input/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.file_input/3","doc":"Generates a file input.\n\nIt requires the given form to be configured with `multipart: true`\nwhen invoking `form_for/4`, otherwise it fails with `ArgumentError`.\n\nSee `text_input/3` for example and docs.","ref":"PhoenixHTMLHelpers.Form.html#file_input/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.form_for/4","doc":"Generates a form tag with a form builder and an anonymous function.\n\n      %>\n      Name:  \n     \n\nForms may be used in two distinct scenarios:\n\n  * with changeset data - when information to populate\n    the form comes from an `Ecto.Changeset` changeset.\n    The changeset holds rich information, which helps\n    provide conveniences\n\n  * with map data - a simple map of parameters (such as\n    `Plug.Conn.params` can be given as data to the form)\n\nWe will explore all them below.\n\nNote that if you are using HEEx templates, `form_for/4` is no longer\nthe preferred way to generate a form tag, and you should use\n[`Phoenix.Component.form/1`](https://hexdocs.pm/phoenix_live_view/Phoenix.Component.html#form/1)\ninstead.","ref":"PhoenixHTMLHelpers.Form.html#form_for/4"},{"type":"function","title":"With changeset data - PhoenixHTMLHelpers.Form.form_for/4","doc":"The entry point for defining forms in Phoenix is with\nthe `form_for/4` function. For this example, we will\nuse `Ecto.Changeset`, which integrates nicely with Phoenix\nforms via the `phoenix_ecto` package.\n\nImagine you have the following action in your controller:\n\n    def new(conn, _params) do\n      changeset = User.changeset(%User{})\n      render conn, \"new.html\", changeset: changeset\n    end\n\nwhere `User.changeset/2` is defined as follows:\n\n    def changeset(user, params \\ %{}) do\n      Ecto.Changeset.cast(user, params, [:name, :age])\n    end\n\nNow a `@changeset` assign is available in views which we\ncan pass to the form:\n\n      %>\n       \n        Name:  \n       \n\n       \n        Age:  \n       \n\n       \n     \n\n`form_for/4` receives the `Ecto.Changeset` and converts it\nto a form, which is passed to the function as the argument\n`f`. All the remaining functions in this module receive\nthe form and automatically generate the input fields, often\nby extracting information from the given changeset. For example,\nif the user had a default value for age set, it will\nautomatically show up as selected in the form.\n\n#","ref":"PhoenixHTMLHelpers.Form.html#form_for/4-with-changeset-data"},{"type":"function","title":"A note on `:errors` - PhoenixHTMLHelpers.Form.form_for/4","doc":"Even if `changeset.errors` is non-empty, errors will not be displayed in a\nform if [the changeset\n`:action`](https://hexdocs.pm/ecto/Ecto.Changeset.html#module-changeset-actions)\nis `nil` or `:ignore`.\n\nThis is useful for things like validation hints on form fields, e.g. an empty\nchangeset for a new form. That changeset isn't valid, but we don't want to\nshow errors until an actual user action has been performed.\n\nFor example, if the user submits and a `Repo.insert/1` is called and fails on\nchangeset validation, the action will be set to `:insert` to show that an\ninsert was attempted, and the presence of that action will cause errors to be\ndisplayed. The same is true for Repo.update/delete.\n\nIf you want to show errors manually you can also set the action yourself,\neither directly on the `Ecto.Changeset` struct field or by using\n`Ecto.Changeset.apply_action/2`. Since the action can be arbitrary, you can\nset it to `:validate` or anything else to avoid giving the impression that a\ndatabase operation has actually been attempted.","ref":"PhoenixHTMLHelpers.Form.html#form_for/4-a-note-on-errors"},{"type":"function","title":"With map data - PhoenixHTMLHelpers.Form.form_for/4","doc":"`form_for/4` expects as first argument any data structure that\nimplements the `Phoenix.HTML.FormData` protocol. By default,\nPhoenix.HTML implements this protocol for `Map`.\n\nThis is useful when you are creating forms that are not backed\nby any kind of data layer. Let's assume that we're submitting a\nform to the `:new` action in the `FooController`:\n\n      %>\n       \n       \n     \n\nOnce the form is submitted, the form contents will be set directly\nas the parameters root, such as `conn.params[\"contents\"]`. If you\nprefer, you can pass the `:as` option to configure them to be nested:\n\n      %>\n       \n       \n     \n\nIn the example above, all form contents are now set inside `conn.params[\"search\"]`\nthanks to the `[as: :search]` option.","ref":"PhoenixHTMLHelpers.Form.html#form_for/4-with-map-data"},{"type":"function","title":"Nested inputs - PhoenixHTMLHelpers.Form.form_for/4","doc":"If your data layer supports embedding or nested associations,\nyou can use `inputs_for` to attach nested data to the form.\n\nImagine the following Ecto schemas:\n\n    defmodule User do\n      use Ecto.Schema\n\n      schema \"users\" do\n        field :name\n        embeds_one :permalink, Permalink\n      end\n\n      def changeset(user \\ %User{}, params) do\n        user\n        |> Ecto.Changeset.cast(params, [:name])\n        |> Ecto.Changeset.cast_embed(:permalink)\n      end\n    end\n\n    defmodule Permalink do\n      use Ecto.Schema\n\n      embedded_schema do\n        field :url\n      end\n    end\n\nIn the form, you can now do this:\n\n      %>\n       \n\n        %>\n         \n       \n     \n\nThe default option can be given to populate the fields if none\nis given:\n\n      %>\n       \n     \n\n`inputs_for/4` can be used to work with single entities or\ncollections. When working with collections, `:prepend` and\n`:append` can be used to add entries to the collection\nstored in the changeset.","ref":"PhoenixHTMLHelpers.Form.html#form_for/4-nested-inputs"},{"type":"function","title":"CSRF protection - PhoenixHTMLHelpers.Form.form_for/4","doc":"CSRF protection is a mechanism to ensure that the user who rendered\nthe form is the one actually submitting it. This module generates a\nCSRF token by default. Your application should check this token on\nthe server to prevent attackers from making requests on your server on\nbehalf of other users. Phoenix checks this token by default.\n\nWhen posting a form with a host in its address, such as \"//host.com/path\"\ninstead of only \"/path\", Phoenix will include the host signature in the\ntoken, and will only validate the token if the accessed host is the same as\nthe host in the token. This is to avoid tokens from leaking to third-party\napplications. If this behaviour is problematic, you can generate a\nnon-host-specific token with `Plug.CSRFProtection.get_csrf_token/0` and\npass it to the form generator via the `:csrf_token` option.","ref":"PhoenixHTMLHelpers.Form.html#form_for/4-csrf-protection"},{"type":"function","title":"Options - PhoenixHTMLHelpers.Form.form_for/4","doc":"* `:as` - the server side parameter in which all params for this\n    form will be collected (i.e. `as: :user_params` would mean all fields\n    for this form will be accessed as `conn.params.user_params` server\n    side). Automatically inflected when a changeset is given.\n\n  * `:method` - the HTTP method. If the method is not \"get\" nor \"post\",\n    an input tag with name `_method` is generated along-side the form tag.\n    Defaults to \"post\".\n\n  * `:multipart` - when true, sets enctype to \"multipart/form-data\".\n    Required when uploading files.\n\n  * `:csrf_token` - for \"post\" requests, the form tag will automatically\n    include an input tag with name `_csrf_token`. When set to false, this\n    is disabled.\n\n  * `:errors` - use this to manually pass a keyword list of errors to the form\n    (for example from `conn.assigns[:errors]`). This option is only used when a\n    connection is used as the form source and it will make the errors available\n    under `f.errors`.\n\n  * `:id` - the ID of the form attribute. If an ID is given, all form inputs\n    will also be prefixed by the given ID.\n\nAll other options will be passed as HTML attributes, such as `class: \"foo\"`.","ref":"PhoenixHTMLHelpers.Form.html#form_for/4-options"},{"type":"function","title":"PhoenixHTMLHelpers.Form.hidden_input/3","doc":"Generates a hidden input.\n\nSee `text_input/3` for example and docs.","ref":"PhoenixHTMLHelpers.Form.html#hidden_input/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.hidden_inputs_for/1","doc":"Generates hidden inputs for the given form inputs.\n\nSee `inputs_for/2` and `inputs_for/3`.","ref":"PhoenixHTMLHelpers.Form.html#hidden_inputs_for/1"},{"type":"function","title":"PhoenixHTMLHelpers.Form.humanize/1","doc":"Converts an attribute/form field into its humanize version.\n\n    iex> humanize(:username)\n    \"Username\"\n    iex> humanize(:created_at)\n    \"Created at\"\n    iex> humanize(\"user_id\")\n    \"User\"","ref":"PhoenixHTMLHelpers.Form.html#humanize/1"},{"type":"function","title":"PhoenixHTMLHelpers.Form.input_type/3","doc":"Gets the input type for a given field.\n\nIf the underlying input type is a `:text_field`,\na mapping could be given to further inflect\nthe input type based solely on the field name.\nThe default mapping is:\n\n    %{\"url\"      => :url_input,\n      \"email\"    => :email_input,\n      \"search\"   => :search_input,\n      \"password\" => :password_input}","ref":"PhoenixHTMLHelpers.Form.html#input_type/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.inputs_for/4","doc":"Generate a new form builder for the given parameter in form.\n\nSee `form_for/4` for examples of using this function.","ref":"PhoenixHTMLHelpers.Form.html#inputs_for/4"},{"type":"function","title":"Options - PhoenixHTMLHelpers.Form.inputs_for/4","doc":"* `:id` - the id to be used in the form, defaults to the\n    concatenation of the given `field` to the parent form id\n\n  * `:as` - the name to be used in the form, defaults to the\n    concatenation of the given `field` to the parent form name\n\n  * `:default` - the value to use if none is available\n\n  * `:prepend` - the values to prepend when rendering. This only\n    applies if the field value is a list and no parameters were\n    sent through the form.\n\n  * `:append` - the values to append when rendering. This only\n    applies if the field value is a list and no parameters were\n    sent through the form.\n\n  * `:skip_hidden` - skip the automatic rendering of hidden\n    fields to allow for more tight control over the generated\n    markup. You can access `form.hidden` to generate them manually\n    within the supplied callback.","ref":"PhoenixHTMLHelpers.Form.html#inputs_for/4-options"},{"type":"function","title":"PhoenixHTMLHelpers.Form.label/1","doc":"Generates a label tag.\n\nUseful when wrapping another input inside a label.","ref":"PhoenixHTMLHelpers.Form.html#label/1"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Form.label/1","doc":"label do\n      radio_button :user, :choice, \"Choice\"\n    end\n    #=>  ... \n\n    label class: \"control-label\" do\n      radio_button :user, :choice, \"Choice\"\n    end\n    #=>  ...","ref":"PhoenixHTMLHelpers.Form.html#label/1-examples"},{"type":"function","title":"PhoenixHTMLHelpers.Form.label/2","doc":"Generates a label tag for the given field.\n\nThe form should either be a `Phoenix.HTML.Form` emitted\nby `form_for` or an atom.\n\nAll given options are forwarded to the underlying tag.\nA default value is provided for `for` attribute but can\nbe overridden if you pass a value to the `for` option.\nText content would be inferred from `field` if not specified\nas either a function argument or string value in a block.\n\nTo wrap a label around an input, see `label/1`.","ref":"PhoenixHTMLHelpers.Form.html#label/2"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Form.label/2","doc":"# Assuming form contains a User schema\n    label(form, :name, \"Name\")\n    #=>  Name \n\n    label(:user, :email, \"Email\")\n    #=>  Email \n\n    label(:user, :email)\n    #=>  Email \n\n    label(:user, :email, class: \"control-label\")\n    #=>  Email \n\n    label :user, :email do\n      \"E-mail Address\"\n    end\n    #=>  E-mail Address \n\n    label :user, :email, \"E-mail Address\", class: \"control-label\"\n    #=>  E-mail Address \n\n    label :user, :email, class: \"control-label\" do\n      \"E-mail Address\"\n    end\n    #=>  E-mail Address","ref":"PhoenixHTMLHelpers.Form.html#label/2-examples"},{"type":"function","title":"PhoenixHTMLHelpers.Form.label/3","doc":"See `label/2`.","ref":"PhoenixHTMLHelpers.Form.html#label/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.label/4","doc":"See `label/2`.","ref":"PhoenixHTMLHelpers.Form.html#label/4"},{"type":"function","title":"PhoenixHTMLHelpers.Form.multiple_select/4","doc":"Generates a select tag with the given `options`.\n\nValues are expected to be an Enumerable containing two-item tuples\n(like maps and keyword lists) or any Enumerable where the element\nwill be used both as key and value for the generated select.","ref":"PhoenixHTMLHelpers.Form.html#multiple_select/4"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Form.multiple_select/4","doc":"# Assuming form contains a User schema\n    multiple_select(form, :roles, [\"Admin\": 1, \"Power User\": 2])\n    #=>  \n    #=>    Admin \n    #=>    Power User \n    #=>  \n\n    multiple_select(form, :roles, [\"Admin\": 1, \"Power User\": 2], selected: [1])\n    #=>  \n    #=>    Admin \n    #=>    Power User \n    #=>  \n\nWhen working with structs, associations, and embeds, you will need to tell\nPhoenix how to extract the value out of the collection. For example,\nimagine `user.roles` is a list of `%Role{}` structs. You must call it as:\n\n    multiple_select(form, :roles, [\"Admin\": 1, \"Power User\": 2],\n                    selected: Enum.map(@user.roles, &(&1.id))\n\nThe `:selected` option will mark the given IDs as selected unless the form\nis being resubmitted. When resubmitted, it uses the form params as values.\n\nWhen used with Ecto, you will typically do a query to retrieve the IDs from\nthe database:\n\n    from r in Role, where: r.id in ^(params[\"roles\"] || [])\n\nAnd then use `Ecto.Changeset.put_assoc/2` to insert the new roles into the user.","ref":"PhoenixHTMLHelpers.Form.html#multiple_select/4-examples"},{"type":"function","title":"Options - PhoenixHTMLHelpers.Form.multiple_select/4","doc":"* `:selected` - the default options to be marked as selected. The values\n     on this list are ignored in case ids have been set as parameters.\n\nAll other options are forwarded to the underlying HTML tag.","ref":"PhoenixHTMLHelpers.Form.html#multiple_select/4-options"},{"type":"function","title":"PhoenixHTMLHelpers.Form.number_input/3","doc":"Generates a number input.\n\nSee `text_input/3` for example and docs.","ref":"PhoenixHTMLHelpers.Form.html#number_input/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.password_input/3","doc":"Generates a password input.\n\nFor security reasons, the form data and parameter values\nare never re-used in `password_input/3`. Pass the value\nexplicitly if you would like to set one.\n\nSee `text_input/3` for example and docs.","ref":"PhoenixHTMLHelpers.Form.html#password_input/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.radio_button/4","doc":"Generates a radio button.\n\nInvoke this function for each possible value you want\nto be sent to the server.","ref":"PhoenixHTMLHelpers.Form.html#radio_button/4"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Form.radio_button/4","doc":"# Assuming form contains a User schema\n    radio_button(form, :role, \"admin\")\n    #=>","ref":"PhoenixHTMLHelpers.Form.html#radio_button/4-examples"},{"type":"function","title":"Options - PhoenixHTMLHelpers.Form.radio_button/4","doc":"All options are simply forwarded to the underlying HTML tag.","ref":"PhoenixHTMLHelpers.Form.html#radio_button/4-options"},{"type":"function","title":"PhoenixHTMLHelpers.Form.range_input/3","doc":"Generates a range input.\n\nSee `text_input/3` for example and docs.","ref":"PhoenixHTMLHelpers.Form.html#range_input/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.reset/2","doc":"Generates a reset input to reset all the form fields to\ntheir original state.\n\nAll options are forwarded to the underlying input tag.","ref":"PhoenixHTMLHelpers.Form.html#reset/2"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Form.reset/2","doc":"reset \"Reset\"\n    #=>  \n\n    reset \"Reset\", class: \"btn\"\n    #=>","ref":"PhoenixHTMLHelpers.Form.html#reset/2-examples"},{"type":"function","title":"PhoenixHTMLHelpers.Form.search_input/3","doc":"Generates a search input.\n\nSee `text_input/3` for example and docs.","ref":"PhoenixHTMLHelpers.Form.html#search_input/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.select/4","doc":"Generates a select tag with the given `options`.\n\n`options` are expected to be an enumerable which will be used to\ngenerate each respective `option`. The enumerable may have:\n\n  * keyword lists - each keyword list is expected to have the keys\n    `:key` and `:value`. Additional keys such as `:disabled` may\n    be given to customize the option.\n\n  * two-item tuples - where the first element is an atom, string or\n    integer to be used as the option label and the second element is\n    an atom, string or integer to be used as the option value\n\n  * atom, string or integer - which will be used as both label and value\n    for the generated select","ref":"PhoenixHTMLHelpers.Form.html#select/4"},{"type":"function","title":"Optgroups - PhoenixHTMLHelpers.Form.select/4","doc":"If `options` is map or keyword list where the first element is a string,\natom or integer and the second element is a list or a map, it is assumed\nthe key will be wrapped in an ` ` and the value will be used to\ngenerate ` ` nested under the group.","ref":"PhoenixHTMLHelpers.Form.html#select/4-optgroups"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Form.select/4","doc":"# Assuming form contains a User schema\n    select(form, :age, 0..120)\n    #=>  \n    #=>    0 \n    #=>   ...\n    #=>    120 \n    #=>  \n\n    select(form, :role, [\"Admin\": \"admin\", \"User\": \"user\"])\n    #=>  \n    #=>    Admin \n    #=>    User \n    #=>  \n\n    select(form, :role, [[key: \"Admin\", value: \"admin\", disabled: true],\n                         [key: \"User\", value: \"user\"]])\n    #=>  \n    #=>    Admin \n    #=>    User \n    #=>  \n\nYou can also pass a prompt:\n\n    select(form, :role, [\"Admin\": \"admin\", \"User\": \"user\"], prompt: \"Choose your role\")\n    #=>  \n    #=>    Choose your role \n    #=>    Admin \n    #=>    User \n    #=>  \n\nAnd customize the prompt like any other entry:\n\n    select(form, :role, [\"Admin\": \"admin\", \"User\": \"user\"], prompt: [key: \"Choose your role\", disabled: true])\n    #=>  \n    #=>    Choose your role \n    #=>    Admin \n    #=>    User \n    #=>  \n\nIf you want to select an option that comes from the database,\nsuch as a manager for a given project, you may write:\n\n    select(form, :manager_id, Enum.map(@managers, &{&1.name, &1.id}))\n    #=>  \n    #=>    Mary Jane \n    #=>    John Doe \n    #=>  \n\nFinally, if the values are a list or a map, we use the keys for\ngrouping:\n\n    select(form, :country, [\"Europe\": [\"UK\", \"Sweden\", \"France\"]], ...)\n    #=>  \n    #=>    \n    #=>      UK \n    #=>      Sweden \n    #=>      France \n    #=>    \n    #=>   ...\n    #=>","ref":"PhoenixHTMLHelpers.Form.html#select/4-examples"},{"type":"function","title":"Options - PhoenixHTMLHelpers.Form.select/4","doc":"* `:prompt` - an option to include at the top of the options. It may be\n    a string or a keyword list of attributes and the `:key`\n\n  * `:selected` - the default value to use when none was sent as parameter\n\nBe aware that a `:multiple` option will not generate a correctly\nfunctioning multiple select element. Use `multiple_select/4` instead.\n\nAll other options are forwarded to the underlying HTML tag.","ref":"PhoenixHTMLHelpers.Form.html#select/4-options"},{"type":"function","title":"PhoenixHTMLHelpers.Form.submit/1","doc":"Generates a submit button to send the form.","ref":"PhoenixHTMLHelpers.Form.html#submit/1"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Form.submit/1","doc":"submit do: \"Submit\"\n    #=>  Submit","ref":"PhoenixHTMLHelpers.Form.html#submit/1-examples"},{"type":"function","title":"PhoenixHTMLHelpers.Form.submit/2","doc":"Generates a submit button to send the form.\n\nAll options are forwarded to the underlying button tag.\nWhen called with a `do:` block, the button tag options\ncome first.","ref":"PhoenixHTMLHelpers.Form.html#submit/2"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Form.submit/2","doc":"submit \"Submit\"\n    #=>  Submit \n\n    submit \"Submit\", class: \"btn\"\n    #=>  Submit \n\n    submit [class: \"btn\"], do: \"Submit\"\n    #=>  Submit","ref":"PhoenixHTMLHelpers.Form.html#submit/2-examples"},{"type":"function","title":"PhoenixHTMLHelpers.Form.telephone_input/3","doc":"Generates a telephone input.\n\nSee `text_input/3` for example and docs.","ref":"PhoenixHTMLHelpers.Form.html#telephone_input/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.text_input/3","doc":"Generates a text input.\n\nThe form should either be a `Phoenix.HTML.Form` emitted\nby `form_for` or an atom.\n\nAll given options are forwarded to the underlying input,\ndefault values are provided for id, name and value if\npossible.","ref":"PhoenixHTMLHelpers.Form.html#text_input/3"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Form.text_input/3","doc":"# Assuming form contains a User schema\n    text_input(form, :name)\n    #=>  \n\n    text_input(:user, :name)\n    #=>","ref":"PhoenixHTMLHelpers.Form.html#text_input/3-examples"},{"type":"function","title":"PhoenixHTMLHelpers.Form.textarea/3","doc":"Generates a textarea input.\n\nAll given options are forwarded to the underlying input,\ndefault values are provided for id, name and textarea\ncontent if possible.","ref":"PhoenixHTMLHelpers.Form.html#textarea/3"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Form.textarea/3","doc":"# Assuming form contains a User schema\n    textarea(form, :description)\n    #=>","ref":"PhoenixHTMLHelpers.Form.html#textarea/3-examples"},{"type":"function","title":"New lines - PhoenixHTMLHelpers.Form.textarea/3","doc":"Notice the generated textarea includes a new line after\nthe opening tag. This is because the HTML spec says new\nlines after tags must be ignored, and all major browser\nimplementations do that.\n\nTherefore, in order to avoid new lines provided by the user\nfrom being ignored when the form is resubmitted, we\nautomatically add a new line before the text area\nvalue.","ref":"PhoenixHTMLHelpers.Form.html#textarea/3-new-lines"},{"type":"function","title":"PhoenixHTMLHelpers.Form.time_input/3","doc":"Generates a time input.","ref":"PhoenixHTMLHelpers.Form.html#time_input/3"},{"type":"function","title":"Options - PhoenixHTMLHelpers.Form.time_input/3","doc":"* `:precision` - Allowed values: `:minute`, `:second`, `:millisecond`.\n    Defaults to `:minute`.\n\nAll other options are forwarded. See `text_input/3` for example and docs.","ref":"PhoenixHTMLHelpers.Form.html#time_input/3-options"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Form.time_input/3","doc":"time_input form, :time\n    #=>  \n\n    time_input form, :time, precision: :second\n    #=>  \n\n    time_input form, :time, precision: :millisecond\n    #=>","ref":"PhoenixHTMLHelpers.Form.html#time_input/3-examples"},{"type":"function","title":"PhoenixHTMLHelpers.Form.time_select/3","doc":"Generates select tags for time.\n\nWarning: This functionality is best provided by browsers nowadays.\nConsider using `time_input/3` instead.\n\nCheck `datetime_select/3` for more information on options and supported values.","ref":"PhoenixHTMLHelpers.Form.html#time_select/3"},{"type":"function","title":"PhoenixHTMLHelpers.Form.url_input/3","doc":"Generates an url input.\n\nSee `text_input/3` for example and docs.","ref":"PhoenixHTMLHelpers.Form.html#url_input/3"},{"type":"module","title":"PhoenixHTMLHelpers.Format","doc":"Formatting functions.","ref":"PhoenixHTMLHelpers.Format.html"},{"type":"function","title":"PhoenixHTMLHelpers.Format.text_to_html/2","doc":"Returns text transformed into HTML using simple formatting rules.\n\nTwo or more consecutive newlines `\\n\\n` or `\\r\\n\\r\\n` are considered as a\nparagraph and text between them is wrapped in ` ` tags.\nOne newline `\\n` or `\\r\\n` is considered as a linebreak and a ` ` tag is inserted.","ref":"PhoenixHTMLHelpers.Format.html#text_to_html/2"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Format.text_to_html/2","doc":"iex> text_to_html(\"Hello\\n\\nWorld\") |> safe_to_string\n    \" Hello \\n World \\n\"\n\n    iex> text_to_html(\"Hello\\nWorld\") |> safe_to_string\n    \" Hello \\nWorld \\n\"\n\n    iex> opts = [wrapper_tag: :div, attributes: [class: \"p\"]]\n    ...> text_to_html(\"Hello\\n\\nWorld\", opts) |> safe_to_string\n    \" Hello \\n World \\n\"","ref":"PhoenixHTMLHelpers.Format.html#text_to_html/2-examples"},{"type":"function","title":"Options - PhoenixHTMLHelpers.Format.text_to_html/2","doc":"* `:escape` - if `false` does not html escape input (default: `true`)\n  * `:wrapper_tag` - tag to wrap each paragraph (default: `:p`)\n  * `:attributes` - html attributes of the wrapper tag (default: `[]`)\n  * `:insert_brs` - if `true` insert ` ` for single line breaks (default: `true`)","ref":"PhoenixHTMLHelpers.Format.html#text_to_html/2-options"},{"type":"module","title":"PhoenixHTMLHelpers.Link","doc":"Conveniences for working with links and URLs in HTML.","ref":"PhoenixHTMLHelpers.Link.html"},{"type":"function","title":"PhoenixHTMLHelpers.Link.button/2","doc":"Generates a button tag that uses the Javascript function handleClick()\n(see phoenix_html.js) to submit the form data.\n\nUseful to ensure that links that change data are not triggered by\nsearch engines and other spidering software.","ref":"PhoenixHTMLHelpers.Link.html#button/2"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Link.button/2","doc":"button(\"hello\", to: \"/world\")\n    #=>  hello \n\n    button(\"hello\", to: \"/world\", method: :get, class: \"btn\")\n    #=>  hello","ref":"PhoenixHTMLHelpers.Link.html#button/2-examples"},{"type":"function","title":"Options - PhoenixHTMLHelpers.Link.button/2","doc":"* `:to` - the page to link to. This option is required\n\n  * `:method` - the method to use with the button. Defaults to :post.\n\nAll other options are forwarded to the underlying button input.\n\nWhen the `:method` is set to `:get` and the `:to` URL contains query\nparameters the generated form element will strip the parameters in accordance\nwith the [W3C](https://www.w3.org/TR/html401/interact/forms.html#h-17.13.3.4)\nform specification.","ref":"PhoenixHTMLHelpers.Link.html#button/2-options"},{"type":"function","title":"Data attributes - PhoenixHTMLHelpers.Link.button/2","doc":"Data attributes are added as a keyword list passed to the\n`data` key. The following data attributes are supported:\n\n  * `data-confirm` - shows a confirmation prompt before generating and\n    submitting the form.","ref":"PhoenixHTMLHelpers.Link.html#button/2-data-attributes"},{"type":"function","title":"PhoenixHTMLHelpers.Link.link/2","doc":"Generates a link to the given URL.","ref":"PhoenixHTMLHelpers.Link.html#link/2"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Link.link/2","doc":"link(\"hello\", to: \"/world\")\n    #=>  hello \n\n    link(\"hello\", to: URI.parse(\"https://elixir-lang.org\"))\n    #=>  hello \n\n    link(\" \", to: \"/world\")\n    #=>  &lt;hello&gt; \n\n    link(\" \", to: \"/world\", class: \"btn\")\n    #=>  &lt;hello&gt; \n\n    link(\"delete\", to: \"/the_world\", data: [confirm: \"Really?\"])\n    #=>  delete \n\n    # If you supply a method other than `:get`:\n    link(\"delete\", to: \"/everything\", method: :delete)\n    #=>  delete \n\n    # You can use a `do ... end` block too:\n    link to: \"/hello\" do\n      \"world\"\n    end\n    #=>  world","ref":"PhoenixHTMLHelpers.Link.html#link/2-examples"},{"type":"function","title":"Options - PhoenixHTMLHelpers.Link.link/2","doc":"* `:to` - the page to link to. This option is required\n\n  * `:method` - the method to use with the link. In case the\n    method is not `:get`, the link is generated inside the form\n    which sets the proper information. In order to submit the\n    form, JavaScript must be enabled\n\n  * `:csrf_token` - a custom token to use for links with a method\n    other than `:get`.\n\nAll other options are forwarded to the underlying ` ` tag.","ref":"PhoenixHTMLHelpers.Link.html#link/2-options"},{"type":"function","title":"Data attributes - PhoenixHTMLHelpers.Link.link/2","doc":"Data attributes are added as a keyword list passed to the `data` key.\nThe following data attributes are supported:\n\n  * `data-confirm` - shows a confirmation prompt before\n    generating and submitting the form when `:method`\n    is not `:get`.","ref":"PhoenixHTMLHelpers.Link.html#link/2-data-attributes"},{"type":"function","title":"CSRF Protection - PhoenixHTMLHelpers.Link.link/2","doc":"By default, CSRF tokens are generated through `Plug.CSRFProtection`.","ref":"PhoenixHTMLHelpers.Link.html#link/2-csrf-protection"},{"type":"module","title":"PhoenixHTMLHelpers.Tag","doc":"Helpers related to producing HTML tags within templates.\n\n> Note: with the addition of the HEEx template engine to\n> Phoenix applications, the functions in this module have\n> lost a bit of relevance and must only be used in special\n> circumstances.\n>\n> Whenever possible, prefer to use the HEEx template engine\n> instead of the functions here. For example, instead of:\n>\n>      \n>       Hello\n>      \n>\n> Do:\n>\n>      \n>       Hello\n>      \n\n> Note: the examples in this module use `safe_to_string/1`\n> imported from `Phoenix.HTML` for readability.","ref":"PhoenixHTMLHelpers.Tag.html"},{"type":"function","title":"PhoenixHTMLHelpers.Tag.content_tag/2","doc":"Creates an HTML tag with given name, content, and attributes.\n\nSee `PhoenixHTMLHelpers.Tag.tag/2` for more information and examples.\n\n    iex> safe_to_string content_tag(:p, \"Hello\")\n    \" Hello \"\n\n    iex> safe_to_string content_tag(:p, \" \", class: \"test\")\n    \" &lt;Hello&gt; \"\n\n    iex> safe_to_string(content_tag :p, class: \"test\" do\n    ...>   \"Hello\"\n    ...> end)\n    \" Hello \"\n\n    iex> safe_to_string content_tag(:option, \"Display Value\", [{:data, [foo: \"bar\"]}, value: \"value\"])\n    \" Display Value \"","ref":"PhoenixHTMLHelpers.Tag.html#content_tag/2"},{"type":"function","title":"PhoenixHTMLHelpers.Tag.content_tag/3","doc":"","ref":"PhoenixHTMLHelpers.Tag.html#content_tag/3"},{"type":"function","title":"PhoenixHTMLHelpers.Tag.csrf_input_tag/2","doc":"Generates a hidden input tag with a CSRF token.\n\nThis could be used when writing a form without the use of tag\nhelpers like `form_tag/3` or `form_for/4`, while maintaining\nCSRF protection.\n\nThe `to` argument should be the same as the form action.","ref":"PhoenixHTMLHelpers.Tag.html#csrf_input_tag/2"},{"type":"function","title":"Example - PhoenixHTMLHelpers.Tag.csrf_input_tag/2","doc":"etc.\n     \n\nAdditional options to the tag can be given.","ref":"PhoenixHTMLHelpers.Tag.html#csrf_input_tag/2-example"},{"type":"function","title":"PhoenixHTMLHelpers.Tag.csrf_meta_tag/1","doc":"Generates a meta tag with CSRF information.\n\nAdditional options to the tag can be given.","ref":"PhoenixHTMLHelpers.Tag.html#csrf_meta_tag/1"},{"type":"function","title":"PhoenixHTMLHelpers.Tag.csrf_token_value/1","doc":"Returns the `csrf_token` value to be used by forms, meta tags, etc.\n\nBy default, CSRF tokens are generated through `Plug.CSRFProtection`\nwhich is capable of generating a separate token per host. Therefore\nit is recommended to pass the `URI` of the destination as argument.\nIf none is given `%URI{host: nil}` is used, which implies a local\nrequest is being done.","ref":"PhoenixHTMLHelpers.Tag.html#csrf_token_value/1"},{"type":"function","title":"PhoenixHTMLHelpers.Tag.form_tag/2","doc":"Generates a form tag.\n\nThis function generates the ` ` tag without its closing part.\nCheck `form_tag/3` for generating an enclosing tag.","ref":"PhoenixHTMLHelpers.Tag.html#form_tag/2"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Tag.form_tag/2","doc":"form_tag(\"/hello\")\n     \n\n    form_tag(\"/hello\", method: :get)","ref":"PhoenixHTMLHelpers.Tag.html#form_tag/2-examples"},{"type":"function","title":"Options - PhoenixHTMLHelpers.Tag.form_tag/2","doc":"* `:method` - the HTTP method. If the method is not \"get\" nor \"post\",\n    an input tag with name `_method` is generated along-side the form tag.\n    Defaults to \"post\".\n\n  * `:multipart` - when true, sets enctype to \"multipart/form-data\".\n    Required when uploading files\n\n  * `:csrf_token` - for \"post\" requests, the form tag will automatically\n    include an input tag with name `_csrf_token`. When set to false, this\n    is disabled\n\nAll other options are passed to the underlying HTML tag.","ref":"PhoenixHTMLHelpers.Tag.html#form_tag/2-options"},{"type":"function","title":"CSRF Protection - PhoenixHTMLHelpers.Tag.form_tag/2","doc":"By default, CSRF tokens are generated through `Plug.CSRFProtection`.","ref":"PhoenixHTMLHelpers.Tag.html#form_tag/2-csrf-protection"},{"type":"function","title":"PhoenixHTMLHelpers.Tag.form_tag/3","doc":"Generates a form tag with the given contents.","ref":"PhoenixHTMLHelpers.Tag.html#form_tag/3"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Tag.form_tag/3","doc":"form_tag(\"/hello\", method: \"get\") do\n      \"Hello\"\n    end\n     ...Hello...","ref":"PhoenixHTMLHelpers.Tag.html#form_tag/3-examples"},{"type":"function","title":"PhoenixHTMLHelpers.Tag.img_tag/2","doc":"Generates an img tag with a src.","ref":"PhoenixHTMLHelpers.Tag.html#img_tag/2"},{"type":"function","title":"Examples - PhoenixHTMLHelpers.Tag.img_tag/2","doc":"img_tag(user.photo_path)\n     \n\n    img_tag(user.photo, class: \"image\")\n     \n\nTo generate a path to an image hosted in your application \"priv/static\",\nwith the `@conn` endpoint, use `static_path/2` to get a URL with\ncache control parameters:\n\n    img_tag(Routes.static_path(@conn, \"/logo.png\"))\n     \n\nFor responsive images, pass a map, list or string through `:srcset`.\n\n    img_tag(\"/logo.png\", srcset: %{\"/logo.png\" => \"1x\", \"/logo-2x.png\" => \"2x\"})\n     \n\n    img_tag(\"/logo.png\", srcset: [\"/logo.png\", {\"/logo-2x.png\", \"2x\"}])","ref":"PhoenixHTMLHelpers.Tag.html#img_tag/2-examples"},{"type":"function","title":"PhoenixHTMLHelpers.Tag.tag/1","doc":"Creates an HTML tag with the given name and options.\n\n    iex> safe_to_string tag(:br)\n    \" \"\n    iex> safe_to_string tag(:input, type: \"text\", name: \"user_id\")\n    \" \"","ref":"PhoenixHTMLHelpers.Tag.html#tag/1"},{"type":"function","title":"Data attributes - PhoenixHTMLHelpers.Tag.tag/1","doc":"In order to add custom data attributes you need to pass\na tuple containing :data atom and a keyword list\nwith data attributes' names and values as the first element\nin the tag's attributes keyword list:\n\n    iex> safe_to_string tag(:input, [data: [foo: \"bar\"], id: \"some_id\"])\n    \" \"","ref":"PhoenixHTMLHelpers.Tag.html#tag/1-data-attributes"},{"type":"function","title":"Boolean values - PhoenixHTMLHelpers.Tag.tag/1","doc":"In case an attribute contains a boolean value, its key\nis repeated when it is true, as expected in HTML, or\nthe attribute is completely removed if it is false:\n\n    iex> safe_to_string tag(:audio, autoplay: \"autoplay\")\n    \" \"\n    iex> safe_to_string tag(:audio, autoplay: true)\n    \" \"\n    iex> safe_to_string tag(:audio, autoplay: false)\n    \" \"\n\nIf you want the boolean attribute to be sent as is,\nyou can explicitly convert it to a string before.","ref":"PhoenixHTMLHelpers.Tag.html#tag/1-boolean-values"},{"type":"function","title":"PhoenixHTMLHelpers.Tag.tag/2","doc":"","ref":"PhoenixHTMLHelpers.Tag.html#tag/2"}],"content_type":"text/markdown"}